@@ -12,15 +12,15 @@
 # License; see this file: https://github.com/RDFLib/rdflib/blob/master/LICENSE
 
 from xmptools.xmptools import makeFileURI, makeFileURI, makeFilePath, XMPMetadata
-from xmptools.xmptools import DC, DCT, XMP, EXIF, CRS, PHOTOSHOP
+from xmptools.xmptools import DC, DCT, XMP, EXIF, CRS, PHOTOSHOP, XMPGIMG
 from xmptools.xmptools import JPEG_EXTENSIONS, TIFF_EXTENSIONS, RAW_EXTENSIONS, DNG_EXTENSIONS
-from xmptools.xmptools import XMP_EXTENSIONS
+from xmptools.xmptools import XMP_EXTENSIONS, PDF_EXTENSIONS
 from xmptools.xmptools import FileTypeError, XMPParser, XMPSerializer
 
 __all__ = [
     'makeFileURI', 'makeFileURI', 'makeFilePath', 'XMPMetadata',
-    'DC', 'DCT', 'XMP', 'EXIF', 'CRS', 'PHOTOSHOP',
+    'DC', 'DCT', 'XMP', 'EXIF', 'CRS', 'PHOTOSHOP', 'XMPGIMG',
     'JPEG_EXTENSIONS', 'TIFF_EXTENSIONS', 'RAW_EXTENSIONS', 'DNG_EXTENSIONS',
-    'XMP_EXTENSIONS',
+    'XMP_EXTENSIONS', 'PDF_EXTENSIONS',
     'FileTypeError', 'XMPParser', 'XMPSerializer'
 ]
