@@ -10,7 +10,7 @@
 # modified because RDFLib did not provide suitable extension points. That code is
 # Copyright (c) 2002-2020, RDFLib Team and is distributed under a similar 3-clause BSD
 # License; see this file: https://github.com/RDFLib/rdflib/blob/master/LICENSE
-
+from base64 import b64decode
 import gzip
 from rdflib import RDF, URIRef, Literal, Namespace, Graph, plugin
 from rdflib.exceptions import Error
@@ -26,6 +26,10 @@ import os.path
 import pathlib
 from urllib.parse import urlparse, unquote_plus
 import rdfhelpers
+from pdfminer.pdfparser import PDFParser
+from pdfminer.pdfdocument import PDFDocument
+from pdfminer.pdftypes import resolve1
+import PIL.Image
 
 TOOLNAME = "So Many Aircraft xmptools 0.4"
 XMP_NS = "adobe:ns:meta/"
@@ -127,6 +131,7 @@ CRS = Namespace("http://ns.adobe.com/camera-raw-settings/1.0/")
 DC = Namespace("http://purl.org/dc/elements/1.1/")
 DCT = Namespace("http://purl.org/dc/terms/")
 PHOTOSHOP = Namespace("http://ns.adobe.com/photoshop/1.0/")
+XMPGIMG = Namespace("http://ns.adobe.com/xap/1.0/g/img/")
 
 class DataNotFound(Error):
     pass
@@ -231,6 +236,7 @@ TIFF_EXTENSIONS = [".tif", ".tiff", ".TIF", ".TIFF"]
 DNG_EXTENSIONS = [".dng", ".DNG"]
 RAW_EXTENSIONS = [".cr2", ".CR2", ".cr3", ".CR3"]  # obviously wholly incomplete still
 XMP_EXTENSIONS = [".xmp", ".XMP"]
+PDF_EXTENSIONS = [".pdf", ".PDF"]
 
 class XMPMetadata(Graph):
     iso8601 = iso8601()
@@ -250,20 +256,26 @@ class XMPMetadata(Graph):
             self.read()
 
     @classmethod
-    def fromFile(cls, path):
+    def fromFile(cls, path, ignore_sidecar_if_pdf=True):
         # LOGIC:
         #   1) Try a corresponding XMP sidecar file
         #   2) Try the image file itself
         #   3) Nothing
-        (base, extension) = os.path.splitext(path)
-        xmppath = base + ".xmp"
-        try:
-            return cls(xmppath), xmppath
-        except (FileNotFoundError, FileTypeError):
+        base, extension = os.path.splitext(path)
+        if ignore_sidecar_if_pdf and (extension in PDF_EXTENSIONS):
+            return cls.fromPDF(path), path
+        else:
+            xmppath = base + ".xmp"
             try:
-                return cls.fromImageFile(path), path
-            except FileTypeError:
-                return None, None
+                return cls(xmppath), xmppath
+            except (FileNotFoundError, FileTypeError):
+                try:
+                    return cls.fromImageFile(path), path
+                except FileTypeError:
+                    try:
+                        return cls.fromPDF(path), path
+                    except FileTypeError:
+                        return None, None
 
     @classmethod
     def fromImageFile(cls, path):
@@ -287,6 +299,23 @@ class XMPMetadata(Graph):
             else:
                 raise FileTypeError("Not a JPEG file: {0}", path)
 
+    @classmethod
+    def fromPDF(cls, path):
+        (_, extension) = os.path.splitext(path)
+        if extension in PDF_EXTENSIONS:
+            with open(path, "rb") as file:
+                doc = PDFDocument(PDFParser(file))
+                if "Metadata" in doc.catalog:
+                    xmp = cls()  # empty graph
+                    xmp.url = makeFileURI(os.path.abspath(path))
+                    xmp.parse(resolve1(doc.catalog["Metadata"]).get_data(),
+                              format="xmp", publicID=xmp.url)
+                    return xmp
+                else:
+                    return None
+        else:
+            raise FileTypeError("Probably not a PDF file: {0}", path)
+
     @classmethod
     def attemptToReadXMP(cls, path, file):
         stuff = file.read()
@@ -435,3 +464,18 @@ class XMPMetadata(Graph):
         _, ext = os.path.splitext(pathname)
         with gzip.open((pathname + ".xmp.gz") if ext == '' else (pathname + ".gz"), "wb") as f:
             self.write(f)
+
+    def getThumbnails(self, predicate=XMP.Thumbnails):
+        thumbs = self.getContainerItems(predicate)
+        if thumbs:
+            thumbnails = list()
+            for thumb in thumbs:
+                with io.BytesIO(b64decode(rdfhelpers.getvalue(self, thumb, XMPGIMG.image))) as f:
+                    with PIL.Image.open(f) as image:
+                        # Must call load() here, because the byte stream is no longer open when the
+                        # image is later accessed; PIL operations are lazy
+                        image.load()
+                        thumbnails.append(image)
+            return thumbnails
+        else:
+            return None
